package com.steerinfo.dil.controller;

import com.steerinfo.dil.model.BmsportHandlingFee;
import com.steerinfo.dil.service.impl.BmsportHandlingFeeServiceImpl;
import com.steerinfo.dil.service.impl.BmsshipStatementServiceImpl;
import com.steerinfo.dil.service.impl.BmstruckFormulaServiceImpl;
import com.steerinfo.dil.util.*;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageHelper;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:HuJianGuo
 * @GreateTime:2021/9/27 17:34
 * @Version:V2.0
 */
@RestController
@RequestMapping("/${api.version}/bmsporthandlingfee")
public class BmsportHandlingFeeController extends BaseRESTfulController {

    @Autowired
    BmsportHandlingFeeServiceImpl bmsportHandlingFeeService;
    @Autowired
    BmsshipStatementServiceImpl bmsshipStatementService;
    @Autowired
    BmstruckFormulaServiceImpl bmstruckFormulaService;
    @Autowired
    ColumnDataUtil columnDataUtil;
    @Autowired
    private ImageFileUtils imageFileUtils;
    private final SimpleDateFormat sdfDateTime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    /**
     * 展示港口装卸费结算信息
     * @param mapValue
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiModelProperty(value = "展示港口装卸费结算信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "151", required = false, dataType = "BigDecimal")
    })
    @PostMapping("/getPortHandlingFeeList")
    public RESTfulResult getPortHandlingFeeList(@RequestBody(required = false) Map<String,Object> mapValue,
                                          Integer pageNum,
                                          Integer pageSize,
                                          Integer apiId) {
        List<Map<String, Object>> list = bmsportHandlingFeeService.getPortHandlingFeeList(mapValue);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmsportHandlingFeeService.getPortHandlingFeeList(mapValue);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, list, columnList);
        return success(data);
    }

    /**
     * 展示批次信息
     * @param mapValue
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiModelProperty(value = "展示批次信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "115", required = false, dataType = "BigDecimal")
    })
    @PostMapping("/getBatch")
    public RESTfulResult getBatch(@RequestBody(required = false) Map<String,Object> mapValue,
                                        Integer pageNum,
                                        Integer pageSize,
                                        Integer apiId) {
        List<Map<String, Object>> list = bmsshipStatementService.getBatch(mapValue);
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmsshipStatementService.getBatch(mapValue);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, list, columnList);
        return success(data);
    }

    /**
     * 新增港口装卸费结算
     * @param map
     * @return
     */
    @ApiModelProperty(value = "新增港口装卸费结算")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "map", value = "港口装卸费字段", required = false, dataType = "java.util.List"),
    })
    @PostMapping("/addPortHandlingFee")
    public RESTfulResult addPortHandlingFee(@RequestBody Map<String,Object> map) throws Exception {
        int code1 = bmsportHandlingFeeService.insert(map);
        return success(code1);
    }




    @ApiModelProperty(value = "修改港口装卸费结算")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "map", value = "港口装卸费字段", required = false, dataType = "java.util.List"),
    })
    @PostMapping("/updatePortHandlingFee")
    public RESTfulResult updatePortHandlingFee(@RequestBody Map<String,Object> map) throws Exception {
        int code1 = bmsportHandlingFeeService.updatePortHandlingFee(map);
        if (code1==-2){
            return  failed("费用不一致，请仔细核对数据是否更改");
        }
        return success(code1);
    }

    /**
     * 新增港口装卸费结算
     * @param map
     * @return
     */
    @ApiModelProperty(value = "新增二程船装卸费用")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "map", value = "港口装卸费字段", required = false, dataType = "java.util.List"),
    })
    @PostMapping("/addPortHandlingFeeSecond")
    public RESTfulResult addPortHandlingFeeSecond(@RequestBody Map<String,Object> map) throws Exception {
        int code1 = bmsportHandlingFeeService.addPortHandlingFeeSecond(map);
        if (code1==-2){
            return failed("费用不一致，请仔细核对数据是否更改");
        }else if(code1==-3){
            return failed("总吨位不一致，请仔细核对数据是否更改");
        }
        return success(code1);
    }

    @ApiModelProperty(value = "展示二程装卸费用")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "map", value = "港口装卸费字段", required = false, dataType = "java.util.List"),
            @ApiImplicitParam(name = "apiId", value = "501", required = false, dataType = "java.lang.Integer"),
    })
    @PostMapping("/selectPortFeeSecondList")
    public RESTfulResult selectPortFeeSecondList(@RequestBody Map<String,Object> map,
                                                 Integer pageNum,
                                                 Integer pageSize,
                                                 Integer apiId,
                                                 Integer feeType,
                                                 String startTime,
                                                 String endTime)
      {
          DataChange.queryDataByDateTime(startTime, endTime, map, sdfDateTime);//根据时间段查询数据
         map.put("feeType",feeType);
         PageHelper.startPage(pageNum, pageSize);
         //分页查询数据
        List<Map<String, Object>> columnList = bmsportHandlingFeeService.selectPortFeeSecondList(map);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, null, columnList);
        return success(data);
    }


    //处理水运单
    @PostMapping(value = "/uploadBmsshipOrder",consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public RESTfulResult uploadBmsshipOrder(@RequestParam("file") MultipartFile multipartFile) throws Exception {
        String url = imageFileUtils.updateFile(multipartFile);
        return success(url);
    }

    //处理水运单
    @PostMapping(value = "/getBmsshipFee/{id}")
    public RESTfulResult getBmsshipFee(@PathVariable("id") Integer id)  {
        List<Map<String,Object>> list=bmsportHandlingFeeService.getBmsshipFee(id);
        return success(list);
    }





    //获取水运单的图片
    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的bmsportHandlingFee信息来更新详细信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "BigDecimal"),
    })
    @PostMapping(value = "/downloadBmsshipOrder", produces  = "application/json;charset=UTF-8")
    public RESTfulResult downloadBmsshipOrder(@RequestParam Integer resultId) {
        String BmsshipOrderPic = bmsportHandlingFeeService.downloadBmsshipOrder(resultId);
        String photo = null;
        try {
            photo = imageFileUtils.downloadFile(BmsshipOrderPic).toString();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return success(photo);
    }


    @ApiModelProperty(value = "逻辑删除港口装卸费结算")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "feeId", value = "主键id", required = false, dataType = "java.util.List"),
    })
    @PostMapping("/deletePortHandlingFeeSecond/{feeId}")
    public RESTfulResult deletePortHandlingFeeSecond(@PathVariable("feeId") BigDecimal feeId){
        int code1 = bmsportHandlingFeeService.deletePortHandlingFeeSecond(feeId);
        return success(code1);
    }



//
//    /**
//     * 修改港口装卸费结算
//     * @param bmsportHandlingFee
//     * @return
//     */
//    @ApiModelProperty(value = "修改港口装卸费结算")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "bmsportHandlingFee", value = "港口装卸费", required = false, dataType = "BmsportHandlingFee"),
//    })
//    @PostMapping("/updatePortHandlingFee")
//    public RESTfulResult updatePortHandlingFee(@RequestBody BmsportHandlingFee bmsportHandlingFee){
//        int code1 = bmsportHandlingFeeService.update(bmsportHandlingFee);
//        return success(code1);
//    }

    /**
     * 查询修改数据渲染表单
     * @param feeId
     * @return
     */
    @ApiOperation(value="查询修改数据渲染表单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "feeId",value = "港口装卸费结算字段",required = true,paramType = "java.math.BigDecimal")
    })
    @PostMapping("/getFeeToUpdate/{feeId}")
    public RESTfulResult getFeeToUpdate(@PathVariable("feeId") BigDecimal feeId) {
        List<Map<String,Object>> tmsshipLoadShipResult = bmsportHandlingFeeService.getFeeToUpdate(feeId);
        return success(tmsshipLoadShipResult);
    }

    /**
     * 逻辑删除港口装卸费结算
     * @param feeId
     * @return
     */
    @ApiModelProperty(value = "逻辑删除港口装卸费结算")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "feeId", value = "主键id", required = false, dataType = "java.util.List"),
    })
    @PostMapping("/deletePortHandlingFee/{feeId}")
    public RESTfulResult deletePortHandlingFee(@PathVariable("feeId") String feeId){
        int code1 = bmsportHandlingFeeService.deleteByUpdate(new BigDecimal(feeId));
        return success(code1);
    }

    /**
     * 获取物资下拉
     * @return
     */
    @GetMapping(value = "/getMaterialId")
    public RESTfulResult getMaterialId(){
        List<Map<String,Object>> list= bmsportHandlingFeeService.getMaterialId();
        return success(list);
    }

    /**
     * 获取价格下拉
     * @return
     */
    @GetMapping(value = "/getPriceId")
    public RESTfulResult getPriceId(){
        List<Map<String,Object>> list= bmsportHandlingFeeService.getPriceId();
        return success(list);
    }



    /**
     * 根据批次查火运湿吨详情
     * @param batchId
     * @return
     */
    @PostMapping("/getTrainDetailsByBatch/{batchId}")
    public RESTfulResult getTrainDetailsByBatch(@PathVariable("batchId")BigDecimal batchId){
        Map<String,Object> result=bmsportHandlingFeeService.getTrainDetailsByBatch(batchId);
        if(result==null)
            return failed("未查询到数据");
        else
            return  success(result);
    }

    /**
     * 根据批次查汽运湿吨详情
     * @param batchId
     * @return
     */
    @PostMapping("/getTruckDetailsByBatch/{batchId}")
    public RESTfulResult getTruckDetailsByBatch(@PathVariable("batchId")BigDecimal batchId){
        Map<String,Object> result=bmsportHandlingFeeService.getTruckDetailsByBatch(batchId);
        if(result==null)
            return failed("未查询到数据");
        else
            return  success(result);
    }


    /**
     * 根据批次查询装船作业
     * @param
     * @return
     */
    @PostMapping("/findLoadByBatch/{batchId}")
    public RESTfulResult findLoadByBatch(@PathVariable("batchId") BigDecimal batchId){
        return success(bmsportHandlingFeeService.findLoadByBatch(batchId));
    }

}
