package com.steerinfo.dil.controller;

import com.alibaba.fastjson.JSON;
import com.steerinfo.dil.feign.ESFeign;
import com.steerinfo.dil.service.impl.BmsshipDetailsOrderSerivceImpl;
import com.steerinfo.dil.util.BaseRESTfulController;
import com.steerinfo.dil.util.ColumnDataUtil;
import com.steerinfo.dil.util.PageListAdd;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageHelper;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiModelProperty;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:HuJianGuo
 * @GreateTime:2021/9/6 17:34
 * @Version:V2.0
 */
@RestController
@RequestMapping("/${api.version}/bmsshipdetailsorder")
public class BmsshipDetailsOrderController extends BaseRESTfulController {

    @Autowired
    BmsshipDetailsOrderSerivceImpl bmsshipDetailsOrderSerivce;
    @Autowired
    ColumnDataUtil columnDataUtil;
    @Autowired
    ESFeign esFeign;

    /**
     * 展示详单信息
     * @param mapValue
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     */
    @ApiModelProperty(value = "展示详单信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "93", required = false, dataType = "BigDecimal")
    })
    @PostMapping("/getDetailsOrderList")
    public RESTfulResult getDetailsOrderList(@RequestBody(required = false) Map<String,Object> mapValue,
                                          Integer pageNum,
                                          Integer pageSize,
                                          Integer apiId)throws IOException {
        //初始化过滤
        List<Map<String, Object>> detailListTotal=null;
        //如果有条件查询则跳过初始化，和创建索引
        if(mapValue==null){
            //将查询结果存入索引中
            detailListTotal = bmsshipDetailsOrderSerivce.getDetailsOrderList(null);
            Map<String, Object> map = new HashMap<>();
            //添加索引
            map.put("index","get_detail_order_list");
            //添加id
            map.put("indexId","detailsId");
            detailListTotal.add(map);
            //新建索引
            String s = JSON.toJSONString(detailListTotal);
            esFeign.insertIndex(detailListTotal);
            //删除
            detailListTotal.remove(detailListTotal.size()-1);
        }
        if (detailListTotal == null) {
            detailListTotal = bmsshipDetailsOrderSerivce.getDetailsOrderList(mapValue);
        }
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmsshipDetailsOrderSerivce.getDetailsOrderList(mapValue);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, detailListTotal, columnList);
        return success(data);
    }

    @PostMapping("/getDetailsOrderList/{apiId}")
    public RESTfulResult getDetailListByCon(@PathVariable("apiId") Integer apiId,
                                            @RequestBody(required=false) Map<String,Object> mapValue,
                                            Integer pageNum,
                                            Integer pageSize,
                                            String con){
        if(mapValue==null){
            mapValue=new HashMap<>();
        }
        //设置要查询的索引名称
        String index="get_detail_order_list";
        //获取查询结果
        PageListAdd list = esFeign.getConResult(mapValue,index,apiId,pageNum,pageSize,con);
        return success(list);
    }

    /**
     * 展示未生成账单的详单信息
     * @param mapValue
     * @param pageNum
     * @param pageSize
     * @param apiId
     * @return
     * @throws IOException
     */
    @ApiModelProperty(value = "展示未生成账单的详单信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer"),
            @ApiImplicitParam(name = "apiId", value = "93", required = false, dataType = "BigDecimal")
    })
    @PostMapping("/getUncomplateDetailsOrderList")
    public RESTfulResult getUncomplateDetailsOrderList(@RequestBody(required = false) Map<String,Object> mapValue,
                                             Integer pageNum,
                                             Integer pageSize,
                                             Integer apiId)throws IOException {
        //初始化过滤
        List<Map<String, Object>> detailListTotal=null;
        //如果有条件查询则跳过初始化，和创建索引
        if(mapValue==null){
            //将查询结果存入索引中
            detailListTotal = bmsshipDetailsOrderSerivce.getUncomplateDetailsOrderList(null);
            Map<String, Object> map = new HashMap<>();
            //添加索引
            map.put("index","get_uncomplate_detail_order_list");
            //添加id
            map.put("indexId","detailsId");
            detailListTotal.add(map);
            //新建索引
            String s = JSON.toJSONString(detailListTotal);
            esFeign.insertIndex(detailListTotal);
            //删除
            detailListTotal.remove(detailListTotal.size()-1);
        }
        if (detailListTotal==null) {
            detailListTotal = bmsshipDetailsOrderSerivce.getUncomplateDetailsOrderList(mapValue);
        }
        PageHelper.startPage(pageNum, pageSize);
        //分页查询数据
        List<Map<String, Object>> columnList = bmsshipDetailsOrderSerivce.getUncomplateDetailsOrderList(mapValue);
        PageListAdd data = columnDataUtil.tableColumnData(apiId, detailListTotal, columnList);
        return success(data);
    }

    @PostMapping("/getUncomplateDetailsOrderList/{apiId}")
    public RESTfulResult getUncomplateDetailsOrderList(@PathVariable("apiId") Integer apiId,
                                            @RequestBody(required=false) Map<String,Object> mapValue,
                                            Integer pageNum,
                                            Integer pageSize,
                                            String con){
        if(mapValue==null){
            mapValue=new HashMap<>();
        }
        //设置要查询的索引名称
        String index="get_uncomplate_detail_order_list";
        //获取查询结果
        PageListAdd list = esFeign.getConResult(mapValue,index,apiId,pageNum,pageSize,con);
        return success(list);
    }

    /**
     * 新增详单
     * @param resultId
     * @return
     */
    @ApiModelProperty(value = "新增详单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "resultId", value = "出厂实绩id", required = false, dataType = "BigDecimal"),
    })
    @PostMapping("/addDetailsOrder/{resultId}")
    public RESTfulResult addDetailsOrder(@PathVariable("resultId") BigDecimal resultId) throws Exception {
        int code = bmsshipDetailsOrderSerivce.insertSelective(resultId);
        return success(code);
    }
}
