package com.steerinfo.dil.service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:HuJianGuo
 * @GreateTime:2021/9/6 17:32
 * @Version:V2.0
 */
public interface IBmstruckFormulaService {

    /**
     * 查询价格日期
     * @return
     */
    List<Double> selectPriceValue(BigDecimal batchId);

    /**
     * 得到月装船比例
     * @param batchId
     * @return
     */
    List<Double> selectShipMentRatio(BigDecimal batchId);

    /**
     * 查询装船日期
     * @param batchId
     * @return
     */
    List<String> selectLoadShipDate(BigDecimal batchId);

    /**
     * 获取到厂湿吨
     * @param resultTotalId
     * @return
     */
    BigDecimal selectArrivalWeight(BigDecimal resultTotalId);

    /**
     * 获取批次id
     * @param resultTotalId
     * @return
     */
    BigDecimal selectBatchId(BigDecimal resultTotalId);

    /**
     * 得到一个批次下的水分质检费总和
     * @param batchId
     * @return
     */
    Double selectWaterFees(BigDecimal batchId);

    /**
     * 根据批次id得到采购订单id
     * @param batchId
     * @return
     */
    BigDecimal selectPurchaseId(BigDecimal batchId);


    /**
     * 替换计算公式并进行计算
     * @param weightTaskResultId
     * @return
     * @throws Exception
     */
    BigDecimal generateDetailedOrder(BigDecimal weightTaskResultId) throws Exception;

    /**
     * 替换途损费计费公式并进行计算
     * @param batchId
     * @return
     * @throws Exception
     */
    BigDecimal generateLossCost(BigDecimal batchId) throws Exception;

    /**
     * 通过出厂实绩查询总实绩id
     * @param resultId
     * @return
     */
    BigDecimal selectTotalId(BigDecimal resultId);

    /**
     * 得到计算因子
     * @param batchId
     * @return
     */
    List<Map<String,Object>> getCalData(BigDecimal batchId);

    /**
     * 查询到厂总湿重
     * @param batchId
     * @return
     */
    BigDecimal getArrivalWeight(BigDecimal batchId);

    /**
     * 得到单价
     * @param batchId
     * @return
     */
    BigDecimal getUnitPrice(BigDecimal batchId);

    /**
     * 根据计重实绩得到计费公式
     * @return
     */
    String selectFormula(BigDecimal formula);

    /**
     * 水分检测计算公式
     * @param mapList
     * @return
     * @throws Exception
     */
    BigDecimal generateWaterFees(List<Map<String,Object>> mapList) throws Exception;

    /**
     * 根据主键查询计费公式
     * @param formulaId
     * @return
     */
    Map<String,Object> selectAnyFormula(BigDecimal formulaId);
}
