package com.steerinfo.dil.service.impl;

import com.steerinfo.dil.mapper.BmstruckFormulaMapper;
import com.steerinfo.dil.service.IBmstruckFormulaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Description:
 * @Author:HuJianGuo
 * @GreateTime:2021/9/6 17:33
 * @Version:V2.0
 */
@Service
public class BmstruckFormulaServiceImpl implements IBmstruckFormulaService {

    @Autowired
    BmstruckFormulaMapper bmstruckFormulaMapper;


    /**
     * 通过出厂实绩得到总实绩id
     * @param resultId
     * @return
     */
    @Override
    public BigDecimal selectTotalId(BigDecimal resultId) {
        return bmstruckFormulaMapper.selectTotalId(resultId);
    }

    /**
     * 得到途损费计算因子
     * @param batchId
     * @return
     */
    @Override
    public List<Map<String, Object>> getCalData(BigDecimal batchId) {
        return bmstruckFormulaMapper.getCalData(batchId);
    }

    /**
     * 得到到厂总湿重
     * @param batchId
     * @return
     */
    @Override
    public BigDecimal getArrivalWeight(BigDecimal batchId) {
        return bmstruckFormulaMapper.getArrivalWeight(batchId);
    }

    /**
     * 得到单价
     * @param batchId
     * @return
     */
    @Override
    public BigDecimal getUnitPrice(BigDecimal batchId) {
        return bmstruckFormulaMapper.getUnitPrice(batchId);
    }

    /**
     * 得到计费公式
     * @param formula
     * @return
     */
    @Override
    public String selectFormula(BigDecimal formula) {
        return bmstruckFormulaMapper.selectFormula(formula);
    }

    /**
     * 替换途损费公式并进行计算
     * @param mapList
     * @return
     * @throws Exception
     */
    public BigDecimal generateWaterFees(List<Map<String,Object>> mapList) throws Exception {
        // 得到船运途损费计费公式
        String formula_string = selectFormula(new BigDecimal(6));
        Integer price = 2000;
        Integer size = mapList.size();
        // 替换船运计算公式
        formula_string = formula_string.replace("单价",price.toString())
                .replace("船数",size.toString());
        String s = toSufExpr(formula_string);
        return calSufExpr(s);
    }

    /**
     * 根据主键查询公式和名称
     * @param formulaId
     * @return
     */
    @Override
    public Map<String, Object> selectAnyFormula(BigDecimal formulaId) {
        return bmstruckFormulaMapper.selectAnyFormula(formulaId);
    }

    /**
     * 替换途损费公式并进行计算
     * @param batchId
     * @return
     * @throws Exception
     */
    public BigDecimal generateLossCost(BigDecimal batchId) throws Exception {
        // 得到船运途损费计费公式
        String formula_string = selectFormula(new BigDecimal(5));
        // 得到单价
        BigDecimal unitPrice = getUnitPrice(batchId);
        // 得到计算因子
        List<Map<String, Object>> calDataList = getCalData(batchId);
        Map<String,Object> calDataMap = calDataList.get(0);
        // 比例
        Double proportion = new Double(0.015);
        // 得到总装船湿吨
        BigDecimal shipWeight = (BigDecimal) calDataMap.get("shipWeight");
        // 得到总装船含水值
        BigDecimal moistureContent = (BigDecimal) calDataMap.get("moistureContent");
        // 得到总到厂湿吨
        BigDecimal arrivalWeight = getArrivalWeight(batchId);
        // 得到质检次数
        BigDecimal count = (BigDecimal) calDataMap.get("count");
        // 得到装船平均水分值
        Double averageContent = moistureContent.doubleValue()/count.intValue();
        DecimalFormat df = new DecimalFormat("0.000");
        String averageMoistureContent = df.format(averageContent.doubleValue());
        // 到厂含水值
        Double arrivalContent = new Double(0.031);
        // 替换船运计算公式
        formula_string = formula_string.replace("装船湿吨",shipWeight.toString())
                .replace("比例",proportion.toString())
                .replace("装船含水值",averageMoistureContent)
                .replace("到厂湿吨",arrivalWeight.toString())
                .replace("到厂含水值",arrivalContent.toString())
                .replace("湿吨单价",unitPrice.toString());
        String s = toSufExpr(formula_string);
        return calSufExpr(s);
    }

    /**
     * 得到到厂湿吨
     * @param resultTotalId
     * @return
     */
    @Override
    public BigDecimal selectArrivalWeight(BigDecimal resultTotalId) {
        return bmstruckFormulaMapper.selectArrivalWeight(resultTotalId);
    }

    /**
     * 根据总实绩id得到批次id
     * @param resultTotalId
     * @return
     */
    @Override
    public BigDecimal selectBatchId(BigDecimal resultTotalId) {
        return bmstruckFormulaMapper.selectBatchId(resultTotalId);
    }

    /**
     * 根据批次id得到采购订单id
     * @param batchId
     * @return
     */
    @Override
    public BigDecimal selectPurchaseId(BigDecimal batchId) {
        return bmstruckFormulaMapper.selectPurchaseId(batchId);
    }

    /**
     * 查询装船日期
     * @param batchId
     * @return
     */
    @Override
    public List<String> selectLoadShipDate(BigDecimal batchId) {
        List<Map<String, Object>> loadShipDateList = bmstruckFormulaMapper.selectLoadShipDate(batchId);
        List<String> yearMonthList = new ArrayList<>();
        for (int i = 0 ; i < loadShipDateList.size();i++) {
            Map<String,Object> loadShipMap = loadShipDateList.get(i);
            Date loadShipDate = (Date) loadShipMap.get("loadShipDate");
            LocalDate localDate = loadShipDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
            Integer year = localDate.getYear();
            Integer month = localDate.getMonthValue();
            // 得到装货日期的年月
            String yearAndMonth = year.toString() + month.toString();
            // 去重
            if (!yearMonthList.contains(yearAndMonth)){
                yearMonthList.add(yearAndMonth);
            }
        }
        return yearMonthList;
    }

    /**
     * 得到价格日期和当月价格
     * @return
     */
    @Override
    public List<Double> selectPriceValue(BigDecimal batchId) {
        // 得到年月
        List<String> yearMonthList = selectLoadShipDate(batchId);
        List<Map<String, Object>> priceDateList = bmstruckFormulaMapper.selectPriceDate();
        List<Double> priceValueList = new ArrayList<>();
        for (int i = 0;i < priceDateList.size();i++){
            Map<String,Object> priceDateMap = priceDateList.get(i);
            Date priceDate = (Date) priceDateMap.get("priceDate");
            LocalDate localDate = priceDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
            Integer year = localDate.getYear();
            Integer month = localDate.getMonthValue();
            // 得到价格日期的年月
            String yearAndMonth = year.toString() + month.toString();
            // 将能与装货年月匹配上的价格存入到list集合
            if (yearMonthList.contains(yearAndMonth)) {
                BigDecimal priceValue = (BigDecimal) priceDateMap.get("priceValue");
                priceValueList.add(priceValue.doubleValue());
            }
        }
        return priceValueList;
    }

    /**
     * 得到跨月装船比例
     * @param batchId
     * @return
     */
    @Override
    public List<Double> selectShipMentRatio(BigDecimal batchId) {
        List<String> yearMonthList = selectLoadShipDate(batchId);
        List<Double> shipMentRatioList = new ArrayList<>();
        // 遍历装船比例，将每个月的装船比例进行累加
        List<Map<String, Object>> loadShipDateList = bmstruckFormulaMapper.selectLoadShipDate(batchId);
        for (int f = 0;f < yearMonthList.size();f++) {
            Double shipmentRatio = new Double(0);
            for (int i = 0;i< loadShipDateList.size();i++) {
                Map<String, Object> ratioMap = loadShipDateList.get(i);
                Date loadShipDate = (Date) ratioMap.get("loadShipDate");
                BigDecimal resultShipmentRatio = (BigDecimal) ratioMap.get("resultShipmentRatio");
                LocalDate localDate = loadShipDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
                Integer year = localDate.getYear();
                Integer month = localDate.getMonthValue();
                String yearAndMonth = year.toString() + month.toString();
                // 判断日期与那个月相匹配，将相匹配的月份装船比例进行累加
                if (yearAndMonth.equals(yearMonthList.get(f))) {
                    shipmentRatio += resultShipmentRatio.doubleValue();
                }
            }
            shipMentRatioList.add(shipmentRatio);
        }
        return shipMentRatioList;
    }

    /**
     * 得到不跨月的月装船比例
     * @param
     * @return
     */
    public Double selectShipMentRatio() {
        return new Double(1);
    }

    /**
     * 得到一个批次下的水分质检费
     * @param batchId
     * @return
     */
    @Override
    public Double selectWaterFees(BigDecimal batchId) {
        return bmstruckFormulaMapper.selectWaterFees(batchId);
    }


    /**
     * 替换船运计费公式并进行计算
     * @param resultId
     * @return
     * @throws Exception
     */
    public BigDecimal generateDetailedOrder(BigDecimal resultId) throws Exception {
        // 得到总实绩id
        BigDecimal resultTotalId = selectTotalId(resultId);
        // 得到船运计费公式
        String formula_string = selectFormula(new BigDecimal(4));
        // 得到批次id
        BigDecimal batchId = selectBatchId(resultTotalId);
        // 得到月价格
        List<Double> priceValueList = selectPriceValue(batchId);
        // 得到月装船比例
        List<Double> shipmentRatio = selectShipMentRatio(batchId);
        // 得到到厂湿吨
        BigDecimal resultNetWeight = selectArrivalWeight(resultTotalId);
        Double arrivalWeight = resultNetWeight.doubleValue();
        // 第一个月装船比例
        Double resultShipmentRatio1 = new Double(0);
        // 第二个月装船比例
        Double resultShipmentRatio2 = new Double(0);
        // 第三个月装船比例
        Double resultShipmentRatio3 = new Double(0);
        // 第四个月装船比例
        Double resultShipmentRatio4 = new Double(0);
        // 第一个月价格
        Double priceValue1 = new Double(0);
        // 第二个月价格
        Double priceValue2 = new Double(0);
        // 第三个月价格
        Double priceValue3 = new Double(0);
        // 第四个月价格
        Double priceValue4 = new Double(0);
        // 判断装船月份值
        Integer size = priceValueList.size();
        if (size == 1) {
            // 不跨月时
            // 得到上月月装船比例
            resultShipmentRatio1 += selectShipMentRatio();
            // 得到上月月价格
            priceValue1 += priceValueList.get(0);
        }
        if (size == 2) {
            // 装两个月时
            // 得到第一个月装船比例
            resultShipmentRatio1 += shipmentRatio.get(0);
            // 得到第二个月装船比例
            resultShipmentRatio2 += shipmentRatio.get(1);
            // 得到第一个月价格
            priceValue1 += priceValueList.get(0);
            // 得到第二个月价格
            priceValue2 += priceValueList.get(1);
        }
        if (size == 3) {
            // 装三个月时
            // 得到第一个月装船比例
            resultShipmentRatio1 += shipmentRatio.get(0);
            // 得到第二个月装船比例
            resultShipmentRatio2 += shipmentRatio.get(1);
            // 得到第三个月装船比例
            resultShipmentRatio3 += shipmentRatio.get(2);
            // 得到第一个月价格
            priceValue1 += priceValueList.get(0);
            // 得到第二个月价格
            priceValue2 += priceValueList.get(1);
            // 得到第三个月价格
            priceValue3 += priceValueList.get(2);
        }
        if (size == 4) {
            // 装三个月时
            // 得到第一个月装船比例
            resultShipmentRatio1 += shipmentRatio.get(0);
            // 得到第二个月装船比例
            resultShipmentRatio2 += shipmentRatio.get(1);
            // 得到第三个月装船比例
            resultShipmentRatio3 += shipmentRatio.get(2);
            // 得到第四个月装船比例
            resultShipmentRatio4 += shipmentRatio.get(3);
            // 得到第一个月价格
            priceValue1 += priceValueList.get(0);
            // 得到第二个月价格
            priceValue2 += priceValueList.get(1);
            // 得到第三个月价格
            priceValue3 += priceValueList.get(2);
            // 得到第四个月价格
            priceValue4 += priceValueList.get(3);
        }
        // 替换船运计算公式
        formula_string = formula_string.replace("到厂湿吨",arrivalWeight.toString())
                .replace("第一月装船比例",resultShipmentRatio1.toString())
                .replace("第一月价格",priceValue1.toString())
                .replace("第二月装船比例",resultShipmentRatio2.toString())
                .replace("第二月价格",priceValue2.toString())
                .replace("第三月装船比例",resultShipmentRatio3.toString())
                .replace("第三月价格",priceValue3.toString())
                .replace("第四月装船比例",resultShipmentRatio4.toString())
                .replace("第四月价格",priceValue4.toString());
        String s = toSufExpr(formula_string);
        return calSufExpr(s);
    }

    /**
     * 替换港口装卸费计费公式并进行计算
     * @param feeMaterialTonnage
     * @param priceId
     * @return
     * @throws Exception
     */
    public BigDecimal generatePortHandlingFee(BigDecimal feeMaterialTonnage,BigDecimal priceId) throws Exception {
        // 得到船运计费公式
        String formula_string = selectFormula(new BigDecimal(9));
        BigDecimal priceValue = bmstruckFormulaMapper.getPriceValue(priceId);
        // 替换船运计算公式
        formula_string = formula_string.replace("港口商检吨位",feeMaterialTonnage.toString())
                .replace("单价",priceValue.toString());
        String s = toSufExpr(formula_string);
        return calSufExpr(s);
    }

    /**
     * 中缀转换后缀表达式
     */
    public  String toSufExpr(String s) {
        StringBuffer sufExpr = new StringBuffer();
        Stack<String> stack = new Stack<>();
        //压入空方便判断
        stack.push(null);
        Pattern pattern = Pattern.compile("\\(int\\)|(?<!\\d|[)])-?\\d+(\\.\\d+)?|!=|>=|<=|==|[+\\-*/%()><=|]");
        Matcher matcher = pattern.matcher(s);
        while (matcher.find()) {
            String group = matcher.group();
            if (group.matches("!=|>=|<=|==|[+\\-*/%()><=|]|\\(int\\)")) {//是运算符
                if (group.equals("(")) {//之间压栈
                    stack.push(group);
                } else if (group.equals(")")) {//等于右括号，一直弹栈，直到遇到左括号
                    String pop = null;
                    while (!(pop = stack.pop()).equals("(")) {//遍历循环至左括号
                        sufExpr.append(pop + " ");
                    }
                } else {//是运算符：加减乘除强转，根据优先级压栈
                    while (getPriority(group) <= getPriority(stack.peek())) {
                        String pop = stack.pop();
                        System.out.println(pop);
                        System.out.println(sufExpr);
                        if(pop!=null&&!pop.equals("|")){
                            sufExpr.append(pop + " ");
                        }
                    }
                    stack.push(group);
                }
            } else {//是数字
                sufExpr.append(group + " ");
            }
        }
        String pop = null;
        while (null != (pop = stack.pop())) {
            sufExpr.append(pop + " ");
        }
        return sufExpr.toString();
    }

    /**
     * 设置出栈优先级，(：优先级最低，表示只有遇到)才能出栈
     * 当栈顶遇到优先级比自己小或等于的弹栈
     */
    public  Integer getPriority(String c) {
        if (c == null) {
            return 0;
        }
        switch (c) {
            case "(":
                return 1;
            case "+":
            case "-":
                return 2;
            case "*":
            case "/":
            case "%":
                return 3;
            case ">":
            case "<":
            case ">=":
            case "<=":
            case "!=":
            case "==":
                return 4;
            case "|":
                return 5;
            case "=":
                return 6;
            case "(int)":
                return 7;
        }
        return 0;
    }

    /**
     * 计算值
     */
    public  BigDecimal cal(BigDecimal a1, BigDecimal a2, String operator) throws Exception {
        switch (operator){
            case "+":return a1.add(a2);
            case "-":return a1.subtract(a2);
            case "*":return a1.multiply(a2);
            case "/":return a1.divide(a2,3,BigDecimal.ROUND_CEILING);
            case "%":return a1.divideAndRemainder(a2)[1];
            default:
                break;
        }
        throw new Exception("非法运算符");
    }

    public  Boolean calBoolean(BigDecimal a1, BigDecimal a2, String operator) throws Exception {
        switch (operator){
            case ">":return a1.compareTo(a2) == 1;
            case "<":return a1.compareTo(a2) == -1;
            case ">=":return a1.compareTo(a2) > -1;
            case "<=":return a1.compareTo(a2) < 1;
            case "!=":return a1.compareTo(a2) != 0;
            case "==":return a1.compareTo(a2) == 0;
            default:
                break;
        }
        throw new Exception("非法运算符");
    }

    /**
     * 计算后缀表达式
     */
    public  BigDecimal calSufExpr(String sufExpr) throws Exception {
        Stack<BigDecimal> stack = new Stack<>();
        Pattern pattern=Pattern.compile("\\(int\\)|-?\\d+(\\.\\d+)?|!=|>=|<=|==|[+\\-*/%><=]");//提取运算符和数字
        Matcher matcher = pattern.matcher(sufExpr);
        BigDecimal result=new BigDecimal("0.0");
        while (matcher.find()){
            String group = matcher.group();
            if(group.matches("[+\\-*/%]")){
                BigDecimal a1 =stack.pop();
                BigDecimal a2 =stack.pop();
                result = cal(a2, a1, group);
                stack.push(result);
            }
            else if(group.equals("(int)")){
                BigDecimal pop = stack.pop();
                BigDecimal bigDecimal = pop.setScale(0, BigDecimal.ROUND_DOWN);
                stack.push(bigDecimal);
            }
            else if(group.matches("!=|>=|<=|==|[><]")){
                BigDecimal a1 =stack.pop();
                BigDecimal a2 =stack.pop();
                stack.push(calBoolean(a2,a1,group)?new BigDecimal(1):new BigDecimal(0));
            }
            else if(group.equals("=")){
                BigDecimal a1 =stack.pop();
                BigDecimal a2 =stack.pop();
                if (a2.compareTo(new BigDecimal(1))==0) {
                    stack.push(a1);
                }
            }
            else {//为数字直接压栈
                stack.push(new BigDecimal(group));
            }
        }
        return result;
    }

}
